package id

import "fmt"

const (
	Locale        = "locale"
	LocalesPrefix = "locales"
)

var _ Descriptor = &LocaleID{}

type LocaleID struct {
	SpaceId
	LocaleID string `json:"locale_id,omitempty" bson:"locale_id,omitempty"`
}

func (id *LocaleID) New() Descriptor {
	return &LocaleID{}
}

func (id *LocaleID) Type() string { return Locale }

func (id *LocaleID) String() string {
	return Join(id.SpaceId.String(), LocalesPrefix, id.LocaleID)
}

func (id *LocaleID) FromParts(parts []string) error {
	if len(parts) != 4 || parts[2] != LocalesPrefix {
		return ErrInvalidID
	}
	if err := id.SpaceId.FromParts(parts[:2]); err != nil {
		return err
	}
	id.LocaleID = parts[3]
	return nil
}

func (id *LocaleID) Map() map[string]any {
	m := id.SpaceId.Map()
	m["locale_id"] = id.LocaleID
	m["type"] = Locale
	return m
}

func (id *LocaleID) FromMap(m map[string]any) error {
	id.LocaleID = m["locale_id"].(string)
	if id.LocaleID == "" {
		return fmt.Errorf("%w: LocaleID required", ErrInvalidID)
	}
	return id.SpaceId.FromMap(m)
}

func (id *LocaleID) Validate() error {
	if id.LocaleID == "" {
		return fmt.Errorf("%w: LocaleID required", ErrInvalidID)
	}

	return id.SpaceId.Validate()
}

func NewLocaleId(spaceID, id string) *ObjectId {
	return &ObjectId{Descriptor: &LocaleID{SpaceId: SpaceId{SpaceID: spaceID}, LocaleID: id}}
}
